@@ -7,7 +7,6 @@ import os
 import socket
 import ssl
 import sys
-from enum import Enum
 from typing import List, Tuple
 
 import click
@@ -116,27 +115,6 @@ def create_ssl_context(
     return ctx
 
 
-class _IPKind(Enum):
-    IPv4 = "IPv4"
-    IPv6 = "IPv6"
-
-
-def _get_server_start_message(
-    host_ip_version: _IPKind = _IPKind.IPv4,
-) -> Tuple[str, str]:
-    if host_ip_version is _IPKind.IPv6:
-        ip_repr = "%s://[%s]:%d"
-    else:
-        ip_repr = "%s://%s:%d"
-    message = f"Uvicorn running on {ip_repr} (Press CTRL+C to quit)"
-    color_message = (
-        "Uvicorn running on "
-        + click.style(ip_repr, bold=True)
-        + " (Press CTRL+C to quit)"
-    )
-    return message, color_message
-
-
 class Config:
     def __init__(
         self,
@@ -363,10 +341,15 @@ class Config:
             loop_setup()
 
     def bind_socket(self):
-        family, sockettype, proto, canonname, sockaddr = socket.getaddrinfo(
-            self.host, self.port, type=socket.SOCK_STREAM
-        )[0]
-        sock = socket.socket(family=family, type=sockettype)
+        family = socket.AF_INET
+        addr_format = "%s://%s:%d"
+
+        if self.host and ":" in self.host:
+            # It's an IPv6 address.
+            family = socket.AF_INET6
+            addr_format = "%s://[%s]:%d"
+
+        sock = socket.socket(family=family)
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         try:
             sock.bind((self.host, self.port))
@@ -375,10 +358,12 @@ class Config:
             sys.exit(1)
         sock.set_inheritable(True)
 
-        if family == socket.AddressFamily.AF_INET6:
-            message, color_message = _get_server_start_message(_IPKind.IPv6)
-        else:
-            message, color_message = _get_server_start_message(_IPKind.IPv4)
+        message = f"Uvicorn running on {addr_format} (Press CTRL+C to quit)"
+        color_message = (
+            "Uvicorn running on "
+            + click.style(addr_format, bold=True)
+            + " (Press CTRL+C to quit)"
+        )
         protocol_name = "https" if self.is_ssl else "http"
         logger.info(
             message,
