@@ -113,7 +113,6 @@ class H11Protocol(asyncio.Protocol):
         self.scope = None
         self.headers = None
         self.cycle = None
-        self.message_event = asyncio.Event()
 
     # Protocol interface
     def connection_made(self, transport):
@@ -146,18 +145,22 @@ class H11Protocol(asyncio.Protocol):
                 # Premature client disconnect
                 pass
 
-        self.message_event.set()
+        if self.cycle is not None:
+            self.cycle.message_event.set()
         if self.flow is not None:
             self.flow.resume_writing()
 
     def eof_received(self):
         pass
 
-    def data_received(self, data):
+    def _unset_keepalive_if_required(self):
         if self.timeout_keep_alive_task is not None:
             self.timeout_keep_alive_task.cancel()
             self.timeout_keep_alive_task = None
 
+    def data_received(self, data):
+        self._unset_keepalive_if_required()
+
         self.conn.receive_data(data)
         self.handle_events()
 
@@ -231,7 +234,7 @@ class H11Protocol(asyncio.Protocol):
                     access_logger=self.access_logger,
                     access_log=self.access_log,
                     default_headers=self.default_headers,
-                    message_event=self.message_event,
+                    message_event=asyncio.Event(),
                     on_response=self.on_response_complete,
                 )
                 task = self.loop.create_task(self.cycle.run_asgi(app))
@@ -244,7 +247,7 @@ class H11Protocol(asyncio.Protocol):
                 self.cycle.body += event.data
                 if len(self.cycle.body) > HIGH_WATER_LIMIT:
                     self.flow.pause_reading()
-                self.message_event.set()
+                self.cycle.message_event.set()
 
             elif event_type is h11.EndOfMessage:
                 if self.conn.our_state is h11.DONE:
@@ -252,7 +255,7 @@ class H11Protocol(asyncio.Protocol):
                     self.conn.start_next_cycle()
                     continue
                 self.cycle.more_body = False
-                self.message_event.set()
+                self.cycle.message_event.set()
 
     def handle_upgrade(self, event):
         upgrade_value = None
@@ -299,6 +302,8 @@ class H11Protocol(asyncio.Protocol):
             return
 
         # Set a short Keep-Alive timeout.
+        self._unset_keepalive_if_required()
+
         self.timeout_keep_alive_task = self.loop.call_later(
             self.timeout_keep_alive, self.timeout_keep_alive_handler
         )
@@ -336,7 +341,8 @@ class H11Protocol(asyncio.Protocol):
 
     def timeout_keep_alive_handler(self):
         """
-        Called on a keep-alive connection if no new data is received after a short delay.
+        Called on a keep-alive connection if no new data is received after a short
+        delay.
         """
         if not self.transport.is_closing():
             event = h11.ConnectionClosed()
@@ -485,6 +491,7 @@ class RequestResponseCycle:
             # Handle response completion
             if not more_body:
                 self.response_complete = True
+                self.message_event.set()
                 event = h11.EndOfMessage()
                 output = self.conn.send(event)
                 self.transport.write(output)
